package order

import (
	logger "eta/eta_mini_ht_api/common/component/log"
	"eta/eta_mini_ht_api/common/exception"
	"eta/eta_mini_ht_api/common/utils/page"
	"eta/eta_mini_ht_api/domian/merchant"
	productDao "eta/eta_mini_ht_api/models/merchant"
	orderDao "eta/eta_mini_ht_api/models/order"
	"fmt"
	"math/rand"
	"sync"
	"time"
)

type ProductOrderDTO struct {
	ID                 int
	OrderID            string
	UserID             int
	TemplateUserID     int
	RealName           string
	TradeNo            string
	AreaCode           string
	Mobile             string
	ProductID          int
	ProductType        string
	TotalAmount        string
	ProductPrice       string
	ProductName        string
	ProductDescription string
	PaymentWay         string
	PaymentTimeRemain  int64
	Status             string
	StatusCN           string
	CreatedTime        string
	UpdatedTime        string
}

type ProductOrderDetailDTO struct {
	ProductOrderDTO
	ValidDuration    string
	TransactionID    int
	PaymentTime      string
	RefundFinishTime string
	RefundStatus     string
	Remark           string
}

var (
	typeTransfer = map[productDao.MerchantProductType]string{
		productDao.Report:  "单篇报告",
		productDao.Video:   "单个视频",
		productDao.Audio:   "单个音频",
		productDao.Package: "套餐",
	}
	transProductOrderStatusMap = map[orderDao.OrderStatus]string{
		orderDao.OrderStatusClosed:  "已关闭",
		orderDao.OrderStatusPaid:    "已支付",
		orderDao.OrderStatusPending: "待支付",
		orderDao.OrderStatusRefund:  "售后",
	}

	transRefundStatusMap = map[orderDao.RefundStatus]string{
		orderDao.RefundStatusPending:    "待退款",
		orderDao.RefundStatusFailed:     "退款失败",
		orderDao.RefundStatusProcessing: "退款中",
		orderDao.RefundStatusSuccess:    "退款成功",
	}
	transProductMap = map[string]productDao.MerchantProductType{
		"report":  productDao.Report,
		"video":   productDao.Video,
		"audio":   productDao.Audio,
		"package": productDao.Package,
	}
	productTransMap = map[productDao.MerchantProductType]string{
		productDao.Package: "package",
		productDao.Video:   "video",
		productDao.Audio:   "audio",
		productDao.Report:  "report",
	}
	productOrderStatusMap = map[string]orderDao.OrderStatus{
		"closed":  orderDao.OrderStatusClosed,
		"paid":    orderDao.OrderStatusPaid,
		"pending": orderDao.OrderStatusPending,
		"refund":  orderDao.OrderStatusRefund,
	}
)

func GenerateProductOrderNo() string {
	timestamp := time.Now().UnixNano() / 1000000 // 毫秒级时间戳
	// 生成随机数
	rand.New(rand.NewSource(time.Now().UnixNano()))
	randomPart := rand.Intn(999999)
	// 格式化订单号
	orderNumber := fmt.Sprintf("P%d%06d", timestamp, randomPart)
	return orderNumber
}

func CreateProductOrder(orderDTO ProductOrderDTO) (orderDBDTO ProductOrderDTO, err error) {
	order := convertProductOrder(orderDTO)
	if order.ProductType == "" {
		err = exception.New(exception.IllegalProductType)
		return
	}
	order, err = orderDao.CreateProductOrder(order)
	orderDBDTO = ConvertProductOrderDTO(order)
	return
}

func ConvertProductOrderDTO(order orderDao.ProductOrder) (orderDTO ProductOrderDTO) {
	orderDTO = ProductOrderDTO{
		ID:             order.ID,
		OrderID:        order.OrderID,
		UserID:         order.UserID,
		TemplateUserID: order.TemplateUserID,
		ProductID:      order.ProductID,
		TradeNo:        order.TradeNO,
		TotalAmount:    order.TotalAmount,
		PaymentWay:     string(order.PaymentWay),
		Status:         string(order.Status),
		StatusCN:       transProductOrderStatusMap[order.Status],
		CreatedTime:    order.CreatedTime.Format(time.DateTime),
	}
	if order.Status == orderDao.OrderStatusRefund {
		orderDTO.Status = string(order.RefundStatus)
		orderDTO.StatusCN = transRefundStatusMap[order.RefundStatus]
	}
	if order.Status == orderDao.OrderStatusPending {
		duration := time.Now().Sub(order.ExpiredTime)
		timeout := duration - 15*time.Minute
		if timeout > 0 {
			logger.Info("订单已超时:%v", order.OrderID)
			orderDTO.Status = string(orderDao.OrderStatusClosed)
			orderDTO.StatusCN = transProductOrderStatusMap[orderDao.OrderStatusClosed]
		} else {
			orderDTO.PaymentTimeRemain = int64(duration.Seconds())
		}
	}
	return
}
func convertProductOrderDetailDTO(order orderDao.ProductOrder) (orderDetailDTO ProductOrderDetailDTO) {
	orderDetailDTO = ProductOrderDetailDTO{
		ProductOrderDTO: ProductOrderDTO{
			ID:             order.ID,
			OrderID:        order.OrderID,
			UserID:         order.UserID,
			TemplateUserID: order.TemplateUserID,
			ProductID:      order.ProductID,
			ProductType:    typeTransfer[order.ProductType],
			TotalAmount:    order.TotalAmount,
			PaymentWay:     string(order.PaymentWay),
			Status:         string(order.Status),
			StatusCN:       transProductOrderStatusMap[order.Status],
			CreatedTime:    order.CreatedTime.Format(time.DateTime),
			Mobile:         order.Mobile,
			RealName:       order.RealName,
		},
		TransactionID: order.TradeId,
		RefundStatus:  string(order.RefundStatus),
		Remark:        order.Remark,
	}
	if order.Status == orderDao.OrderStatusRefund {
		if order.RefundStatus == orderDao.RefundStatusSuccess {
			orderDetailDTO.RefundFinishTime = order.RefundFinishTime.Format(time.DateTime)
		}
		orderDetailDTO.StatusCN = transRefundStatusMap[order.RefundStatus]
	}
	if order.Status == orderDao.OrderStatusPending {
		duration := time.Now().Sub(order.ExpiredTime)
		timeout := duration - 15*time.Minute
		if timeout > 0 {
			logger.Info("订单已超时:%v", order.OrderID)
			orderDetailDTO.Status = string(orderDao.OrderStatusClosed)
			orderDetailDTO.StatusCN = transProductOrderStatusMap[orderDao.OrderStatusClosed]
		} else {
			orderDetailDTO.PaymentTimeRemain = int64(duration.Seconds())
			product, err := productDao.GetMerchantProductById(order.ProductID)
			if err != nil {
				logger.Error("获取产品信息失败:%v,productId:%d", err, order.ProductID)
			}
			if product.Type != productDao.Package {
				orderDetailDTO.ValidDuration = "永久有效"
			} else {
				beginDate := time.Now()
				endDate := time.Now().Add(time.Duration(product.ValidDays * 24))
				orderDetailDTO.ValidDuration = fmt.Sprintf("%s~%s", beginDate.Format(time.DateOnly), endDate.Format(time.DateOnly))
			}
		}
	}
	if order.Status == orderDao.OrderStatusPaid {
		access, err := merchant.GetUserSubscribe(order.TemplateUserID, order.ProductID)
		if err != nil {
			logger.Error("获取用户订阅信息失败:%v，templateUserId:%d,productId :%d", err, order.TemplateUserID, order.ProductID)
		} else {
			if access.ProductType == productTransMap[productDao.Package] {
				orderDetailDTO.ValidDuration = "永久有效"
			} else {
				orderDetailDTO.ValidDuration = fmt.Sprintf("%s~%s", access.BeginDate, access.EndDate)
			}
		}
	}
	if order.TradeNO != "" {
		orderDetailDTO.PaymentTime = order.PaymentTime.Format(time.DateTime)
	}
	return
}

func convertProductOrder(order ProductOrderDTO) orderDao.ProductOrder {
	return orderDao.ProductOrder{
		ID:             order.ID,
		OrderID:        order.OrderID,
		UserID:         order.UserID,
		TemplateUserID: order.TemplateUserID,
		RealName:       order.RealName,
		AreaCode:       order.AreaCode,
		Mobile:         order.Mobile,
		ProductID:      order.ProductID,
		ProductType:    transProductMap[order.ProductType],
		ProductName:    order.ProductName,
		TotalAmount:    order.TotalAmount,
	}
}

func GetTotalPageCountByUserId(userId int, orderStatus string) (total int64, latestId int64) {
	var orderStatusEnum orderDao.OrderStatus
	if orderStatus != "" {
		orderStatusEnum = productOrderStatusMap[orderStatus]
		if string(orderStatusEnum) == "" {
			//err := exception.New(exception.IllegalOrderStatus)
			logger.Error("订单状态不合法:%v", orderStatus)
			return
		}
	}
	return orderDao.GetTotalPageCountByUserId(userId, orderStatusEnum)
}
func GetOrderPage(pageInfo page.PageInfo, userId int, orderStatus string) (orderDTOS []ProductOrderDTO, err error) {
	var orderStatusEnum orderDao.OrderStatus
	if orderStatus != "" {
		orderStatusEnum = productOrderStatusMap[orderStatus]
		if string(orderStatusEnum) == "" {
			err = exception.New(exception.IllegalOrderStatus)
			return
		}
	}
	offset := page.StartIndex(pageInfo.Current, pageInfo.PageSize)
	orderList, err := orderDao.GetOrderPage(pageInfo.LatestId, offset, pageInfo.PageSize, userId, orderStatusEnum)
	if err != nil {
		logger.Error("分页查询订单列表失败:%v", err)
		return
	}
	for _, order := range orderList {
		orderDTOS = append(orderDTOS, ConvertProductOrderDTO(order))
	}
	return
}

func GetUserOrderByProduct(productId int, userId int, status string) (orderDTO ProductOrderDTO, err error) {
	order, err := orderDao.GetUserOrderByProduct(productId, userId, productOrderStatusMap[status])
	if err != nil {
		logger.Error("查询订单详情失败:%v", err)
		return
	}
	product, err := productDao.GetMerchantProductById(order.ProductID)
	if err != nil {
		logger.Error("获取产品信息失败:%v", err)
		return
	}
	orderDTO = ConvertProductOrderDTO(order)
	orderDTO.ProductName = product.Title
	orderDTO.ProductDescription = product.Description
	return
}
func GetOrderDetail(orderId string, userId int) (orderDTO ProductOrderDetailDTO, err error) {
	order, err := orderDao.GetOrderDetail(orderId, userId)
	if err != nil {
		logger.Error("查询订单详情失败:%v", err)
		return
	}
	product, err := productDao.GetMerchantProductById(order.ProductID)
	if err != nil {
		logger.Error("获取产品信息失败:%v", err)
		return
	}
	orderDTO = convertProductOrderDetailDTO(order)
	orderDTO.ProductName = product.Title
	orderDTO.ProductDescription = product.Description
	return
}
func BatchCloseOrder() (err error) {
	var total int
	var minId int
	total, minId = orderDao.GetUnPendingOrderCount()
	var ids []int
	for total > 0 {
		ids = orderDao.GetExpiredPendingOrderIds(minId, 500)
		var wg sync.WaitGroup
		chunks := chunkSlice(ids, 100)
		wg.Add(len(chunks))
		for _, chunk := range chunks {
			go func(ids []int) {
				defer wg.Done()
				err = orderDao.BatchCloseOrder(ids)
			}(chunk)
		}
		wg.Wait()
		total, minId = orderDao.GetUnPendingOrderCount()
	}
	return
}

// ChunkSlice 将切片分块
func chunkSlice[T any](slice []T, chunkSize int) [][]T {
	if chunkSize <= 0 {
		return nil
	}

	var chunks [][]T
	n := len(slice)
	for i := 0; i < n; i += chunkSize {
		end := i + chunkSize
		if end > n {
			end = n
		}
		chunks = append(chunks, slice[i:end])
	}
	return chunks
}

func GetOrderByUser(templateUserId int, orderNo string) (orderDTO ProductOrderDTO, err error) {
	var order orderDao.ProductOrder
	order, err = orderDao.GetOrderByUser(templateUserId, orderNo)
	product, err := productDao.GetMerchantProductById(order.ProductID)
	if err != nil {
		logger.Error("获取产品信息失败:%v", err)
		return
	}
	orderDTO = ConvertProductOrderDTO(order)
	orderDTO.ProductName = product.Title
	orderDTO.ProductDescription = product.Description
	return
}

func CloseProductOrder(templateUserId int, productOrderNo string) (err error) {
	return orderDao.CloseProductOrder(templateUserId, productOrderNo)
}
