package merchant

import (
	logger "eta/eta_mini_ht_api/common/component/log"
	"eta/eta_mini_ht_api/models"
	"gorm.io/gorm"
	"gorm.io/gorm/clause"
	"time"
)

type SubscribeStatus string

const (
	SubscribeValid   SubscribeStatus = "valid"
	SubscribeExpired SubscribeStatus = "expired"
	SubscribeClose   SubscribeStatus = "closed"
)

var (
	subscribeStatusMap = map[SubscribeStatus]OpenType{
		SubscribeValid:   OpenTypeValid,
		SubscribeExpired: OpenTypeExpired,
		SubscribeClose:   OpenTypeClose,
	}
)

// UserSubscriptionAccessList 用户订阅访问列表
type UserSubscriptionAccessList struct {
	ID             int                 `gorm:"column:id;primaryKey"`
	TemplateUserId int                 `gorm:"column:template_user_id"`
	ProductID      int                 `gorm:"column:product_id"`
	ProductName    string              `gorm:"column:product_name"`
	ProductType    MerchantProductType `gorm:"column:product_type"`
	BeginDate      time.Time           `gorm:"column:begin_date"`
	EndDate        time.Time           `gorm:"column:end_date"`
	Status         SubscribeStatus     `gorm:"column:status;type:enum('valid','expired');default:'valid'"`
	CreatedTime    time.Time           `gorm:"column:created_time"`
	UpdatedTime    time.Time           `gorm:"column:updated_time"`
	ProductOrderNo string              `gorm:"-"`
}

func (UserSubscriptionAccessList) TableName() string {
	return "user_subscription_access_list"
}

func GetUserSubscribe(productIds []int, userId int) (list []UserSubscriptionAccessList, err error) {
	db := models.Main()
	err = db.Select("id,template_user_id,product_id,product_name,begin_date,end_date,status").Where("template_user_id=? and product_id in ?", userId, productIds).Find(&list).Error
	return
}

func (access *UserSubscriptionAccessList) BeforeCreate(_ *gorm.DB) (err error) {
	access.CreatedTime = time.Now()
	flow := UserSubscriptionAccessFlow{
		ProductID:      access.ProductID,
		ProductType:    string(access.ProductType),
		ProductOrderId: access.ProductOrderNo,
		OpenType:       subscribeStatusMap[access.Status],
		CreatedTime:    time.Now(),
	}
	_ = CreateAccessFlow(flow)
	return
}

func (access *UserSubscriptionAccessList) BeforeUpdate(_ *gorm.DB) (err error) {
	access.UpdatedTime = time.Now()
	flow := UserSubscriptionAccessFlow{
		TemplateUserId: access.TemplateUserId,
		ProductID:      access.ProductID,
		ProductType:    access.ProductName,
		ProductOrderId: access.ProductOrderNo,
		OpenType:       subscribeStatusMap[access.Status],
		CreatedTime:    time.Now(),
	}
	_ = CreateAccessFlow(flow)
	return
}
func InsertOrUpdateUserSubscribe(access UserSubscriptionAccessList) (err error) {
	db := models.Main()
	OnConflictFunc := clause.OnConflict{
		Columns:   []clause.Column{{Name: "template_user_id"}, {Name: "product_id"}},
		DoUpdates: clause.AssignmentColumns([]string{"status", "begin_date", "end_date", "product_name"}),
	}
	// 执行批量插入或更新操作
	err = db.Clauses(OnConflictFunc).Create(&access).Error
	return
}

func CloseUserAccess(access UserSubscriptionAccessList) (err error) {
	db := models.Main()
	err = db.Model(&access).Where("template_user_id=? AND product_id=?", access.TemplateUserId, access.ProductID).Update("status", SubscribeClose).Error
	// 执行批量插入或更新操作
	return
}
func ExpireUserAccess(access UserSubscriptionAccessList) (err error) {
	db := models.Main()
	err = db.Model(&access).Where("template_user_id=? AND product_id=?", access.TemplateUserId, access.ProductID).Update("status", SubscribeExpired).Error
	// 执行批量插入或更新操作
	return
}
func SubscribeList(templateUserId int, productType string, latestId int64, offset int, pageSize int) (list []UserSubscriptionAccessList, err error) {
	db := models.Main()
	err = db.Model(&UserSubscriptionAccessList{}).Where("id<=? and template_user_id=? AND product_type=? and status=?", latestId, templateUserId, productType, SubscribeValid).Order("begin_date desc").Offset(offset).Limit(pageSize).Find(&list).Error
	return
}
func GetUserSubscribeById(templateUserId int, productId int) (list UserSubscriptionAccessList, err error) {
	db := models.Main()
	err = db.Select("id,template_user_id,product_id,product_name,begin_date,end_date,status").Where(" template_user_id=? AND product_id=? and status=?", templateUserId, productId, SubscribeValid).First(&list).Error
	return
}
func GetTotalUserPageCountByProductType(productType string, templateUserId int) (count int64, latestId int64) {
	db := models.Main()
	err := db.Model(&UserSubscriptionAccessList{}).Select("count(*)").Where("template_user_id=? AND product_type=? and status=?", templateUserId, productType, SubscribeValid).Scan(&count).Error
	if err != nil {
		logger.Error("查询用户订阅总数失败:%s", err)
		return 0, 0
	}
	err = db.Model(&UserSubscriptionAccessList{}).Select("max(id)").Where("template_user_id=? AND product_type=? and status=?", templateUserId, productType, SubscribeValid).Scan(&latestId).Error
	if err != nil {
		logger.Error("查询用户订阅最新id失败:%s", err)
		return 0, 0
	}
	return
}

func GetNeedExpiredAccessCount(deadLine string) (count int) {
	db := models.Main()
	err := db.Model(&UserSubscriptionAccessList{}).Select("count(*)").Where(" product_type=? and status=? and end_date <=?", Package, SubscribeValid, deadLine).Scan(&count).Error
	if err != nil {
		logger.Error("查询用户订阅总数失败:%s", err)
	}
	return
}

func GetExpiredAccessList(num int, deadLine string) (list []UserSubscriptionAccessList, err error) {
	db := models.Main()
	err = db.Model(&UserSubscriptionAccessList{}).Select("*").Where(" product_type=? and status=? and end_date <=? limit 0,?", Package, SubscribeValid, deadLine, num).Find(&list).Error
	if err != nil {
		logger.Error("查询用户订阅总数失败:%s", err)
	}
	return
}
