package order

import (
	"errors"
	logger "eta/eta_mini_ht_api/common/component/log"
	"eta/eta_mini_ht_api/models"
	merchantDao "eta/eta_mini_ht_api/models/merchant"
	"fmt"
	"github.com/go-sql-driver/mysql"
	"gorm.io/gorm"
	"time"
)

type OrderStatus string
type RefundStatus string

const (
	//订单状态
	OrderStatusPending    OrderStatus = "pending"
	OrderStatusProcessing OrderStatus = "processing"
	OrderStatusPaid       OrderStatus = "paid"
	OrderStatusClosed     OrderStatus = "closed"
	OrderStatusRefund     OrderStatus = "refund"
	//退款状态
	RefundStatusSuccess    RefundStatus = "success"
	RefundStatusPending    RefundStatus = "pending"
	RefundStatusFailed     RefundStatus = "failed"
	RefundStatusProcessing RefundStatus = "processing"

	detailColumn = "id,order_id,user_id,template_user_id,real_name,area_code,mobile,product_id,product_type,product_name,status,created_time,updated_time,total_amount,expired_time,valid_duration"
)

type ProductOrder struct {
	ID               int                             `gorm:"column:id;primaryKey;autoIncrement" json:"id"`
	OrderID          string                          `gorm:"column:order_id;size:255;comment:'订单编号'" json:"order_id"`
	UserID           int                             `gorm:"column:user_id;default:null;comment:'用户id'" json:"user_id"`
	TemplateUserID   int                             `gorm:"column:template_user_id;default:null;comment:'临时用户id'" json:"template_user_id"`
	RealName         string                          `gorm:"column:real_name;default:null;comment:'姓名'" json:"real_name"`
	AreaCode         string                          `gorm:"column:area_code;default:null;comment:'手机区号'" json:"area_code"`
	Mobile           string                          `gorm:"column:mobile;default:null;comment:'手机号'" json:"mobile"`
	ProductID        int                             `gorm:"column:product_id;default:null;comment:'产品id'" json:"product_id"`
	ProductType      merchantDao.MerchantProductType `gorm:"column:product_type;default:null;comment:'产品类型'" json:"product_type"`
	ProductName      string                          `gorm:"column:product_name;default:null;comment:'商品名称'" json:"product_name"`
	TotalAmount      string                          `gorm:"column:total_amount;size:255;default:null;comment:'金额'" json:"total_amount"`
	TradeId          int                             `gorm:"column:trade_id;default:null;comment:'支付订单'" json:"trade_id"`
	TradeNO          string                          `gorm:"column:trade_no;default:null;comment:'支付订单'" json:"trade_no"`
	PaymentAmount    string                          `gorm:"column:payment_amount;;comment:'支付金额'"`
	PaymentWay       PaymentWay                      `gorm:"column:payment_way;enum('wechat','alipay');default:null;comment:'支付渠道'"`
	PaymentTime      time.Time                       `gorm:"column:payment_time;default:null;comment:'支付时间'" json:"payment_time"`
	ExpiredTime      time.Time                       `gorm:"column:expired_time;default:null;comment:'超时时间'" json:"expired_time"`
	Status           OrderStatus                     `gorm:"column:status;type:enum('pending','processing','paid','closed','refund');default:'pending';comment:'订单状态'" json:"status"`
	RefundTradeId    string                          `column:"refund_trade_id;null;comment:'退款金额'" json:"refund_trade_id"`
	RefundAmount     string                          `gorm:"column:refund_amount;size:255;default:null;comment:'退款金额'" json:"refund_amount"`
	RefundStatus     RefundStatus                    `gorm:"column:refund_status;type:enum('pending','processing','failed','success');default:null;comment:'退款状态'" json:"refund_status"`
	RefundFinishTime time.Time                       `gorm:"column:refund_finish_time;default:null;comment:'退款完成时间'" json:"refund_finish_time"`
	Remark           string                          `gorm:"column:remark;size:255;default:null;comment:'备注'" json:"remark"`
	ValidDuration    string                          `gorm:"column:valid_duration;default:null;comment:'订单有效期'" json:"valid_duration"`
	IsDeleted        int                             `gorm:"column:is_deleted;size:1;default:0;comment:'是否删除'" json:"is_deleted"`
	CreatedTime      time.Time                       `gorm:"column:created_time;default:null;comment:'创建时间'" json:"created_time"`
	UpdatedTime      time.Time                       `gorm:"column:updated_time;default:null;comment:'更新时间'" json:"updated_time"`
}

func (pr *ProductOrder) BeforeCreate(db *gorm.DB) (err error) {
	pr.Status = OrderStatusPending
	createTime := time.Now()
	expiredTime := createTime.Add(time.Duration(15) * time.Minute)
	pr.CreatedTime = createTime
	pr.ExpiredTime = expiredTime
	pr.IsDeleted = 0
	return
}
func CreateProductOrder(order ProductOrder) (orderDB ProductOrder, err error) {
	db := models.Main()
	err = db.Create(&order).Error
	var mysqlErr *mysql.MySQLError
	if errors.As(err, &mysqlErr) && mysqlErr.Number == 1062 {
		err = gorm.ErrDuplicatedKey
	}
	orderDB = order
	return
}

func GetTotalPageCountByUserId(userId int, status OrderStatus) (total int64, latestId int64) {
	db := models.Main()
	query := db.Model(&ProductOrder{}).Select("count(*)").Where("template_user_id = ? and is_deleted=0 ", userId)
	if string(status) != "" {
		query.Where("  status=?", status)
	}
	err := query.Scan(&total).Error
	if err != nil {
		logger.Error("查询订单总数失败:%v", err)
		return 0, 0
	}
	queryLatestId := db.Model(&ProductOrder{}).Select("MAX(id)").Where("template_user_id = ? and is_deleted=0", userId)
	if string(status) != "" {
		query.Where("  status=?", status)
	}
	err = queryLatestId.Scan(&latestId).Error
	if err != nil {
		logger.Error("查询订单最新订单失败:%v", err)
		return 0, 0
	}
	return
}

func GetOrderPage(latestId int64, offset int, size int, userId int, status OrderStatus) (orderList []ProductOrder, err error) {
	db := models.Main()
	query := db.Model(&ProductOrder{}).Select("*")
	if string(status) != "" {
		query.Where("  status=?", status)
	}
	query.Where("id<=? and template_user_id = ? and is_deleted=0 ", latestId, userId)
	err = query.Order("created_time desc").Offset(offset).Limit(size).Find(&orderList).Error
	if err != nil {
		logger.Error("查询订单总数失败:%v", err)
	}
	return
}

func GetOrderDetail(orderId string, userId int) (order ProductOrder, err error) {
	db := models.Main()
	err = db.Model(&ProductOrder{}).Select("*").Where("order_id = ? and template_user_id = ? and is_deleted=0 ", orderId, userId).First(&order).Error
	return
}
func GetUserOrderByProduct(productId int, userId int, status OrderStatus) (order ProductOrder, err error) {
	db := models.Main()
	err = db.Model(&ProductOrder{}).Select("*").Where("product_id = ? and template_user_id = ? and is_deleted=0 and status =?", productId, userId, status).First(&order).Error
	return
}

func GetUnPendingOrderCount() (count int, minId int) {
	db := models.Main()
	_ = db.Model(&ProductOrder{}).Select("count(*)").Where("is_deleted=0 and status=? and expired_time <= ?", OrderStatusPending, time.Now()).Scan(&count).Error
	if count > 0 {
		_ = db.Model(&ProductOrder{}).Select("MIN(id)").Where("is_deleted=0 and status =? and expired_time <= ?", OrderStatusPending, time.Now()).Scan(&minId).Error
	}
	return
}

func GetExpiredPendingOrderIds(minId int, limit int) (ids []int) {
	db := models.Main()
	_ = db.Model(&ProductOrder{}).Select("id").Where("id>= ? and status=? and  is_deleted=0 and expired_time <= ? order by id asc", minId, OrderStatusPending, time.Now()).Limit(limit).Scan(&ids).Error
	return
}

func BatchCloseOrder(ids []int) (err error) {
	db := models.Main()
	err = db.Model(&ProductOrder{}).Where("expired_time < ?", time.Now()).Where("id in ?", ids).Update("status", OrderStatusClosed).Error
	if err != nil {
		logger.Error("批量关闭订单失败:%v", err)
	}
	return nil
}

func GetOrderByUser(templateUserId int, orderNo string) (order ProductOrder, err error) {
	db := models.Main()
	err = db.Model(&ProductOrder{}).Select("*").Where("template_user_id= ? and order_id=? and is_deleted=0 ", templateUserId, orderNo).First(&order).Error
	return
}
func GetOrderByOrderNo(orderNo string) (order ProductOrder, err error) {
	db := models.Main()
	err = db.Model(&ProductOrder{}).Select("*").Where("order_id=? and is_deleted=0 ", orderNo).First(&order).Error
	return
}
func CloseProductOrder(templateUserId int, productOrderNo string) (err error) {
	db := models.Main()
	var dbOrder ProductOrder
	err = db.Model(&ProductOrder{}).Select("*").Where("template_user_id= ? and order_id=? and is_deleted=0 ", templateUserId, productOrderNo).First(&dbOrder).Error
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			logger.Warn("当前用户[template_user_id:%d]不存在订单编号为%s的订单", templateUserId, productOrderNo)
			err = errors.New(fmt.Sprintf("当前用户[template_user_id:%d]不存在订单编号为%s的订单", templateUserId, productOrderNo))
			return
		}
		logger.Error("获取订单信息失败:%v", err)
	}
	if dbOrder.Status == OrderStatusPaid {
		logger.Error("订单已支付,无法关闭订单[order_id:%s]", productOrderNo)
		err = errors.New("订单已支付,无法关闭订单")
		return
	}
	if dbOrder.Status == OrderStatusRefund {
		logger.Error("订单已售后,无法关闭订单[order_id:%s]", productOrderNo)
		err = errors.New("订单已售后,无法关闭订单")
		return
	}
	err = db.Model(&ProductOrder{}).Where("template_user_id= ? and order_id=? and is_deleted=0 ", templateUserId, productOrderNo).Update("status", OrderStatusClosed).Error
	return
}
